; %W% %G%

; Tom's incredible emacs debugger interface

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; displaying source

(defvar tdb-extent nil)
(make-face 'tdb-face)

(set-face-foreground 'tdb-face "black")
(set-face-background 'tdb-face "LightYellow")

(defun tdb-undo ()
  (cond (tdb-extent
	 (delete-extent tdb-extent)
	 (setq tdb-extent nil))))

(defun tdb (f beg0 end0)
  (tdb-undo)
  (let ((beg (max 1 beg0)) (end (+ end0 1)) (local (get-buffer "*prolog*")))
    (if local (find-file-other-window f) (find-file f))
    (goto-char (+ beg (/ (- end beg) 2)))
    (recenter)
    (goto-char (1+ beg))
    (setq tdb-extent (make-extent beg end))
    (set-extent-face tdb-extent 'tdb-face)
    (if local (pop-to-buffer "*prolog*" nil))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; since Prolog may start up multiple times in the same buffer, it can
; get its notion of where it is in the file confused. So we maintain an
; offset.

(defvar tdb-offset 0)

(defun tdb-character-count (C)
  (setq tdb-offset (- (point) C)))

(defun tdb-make-extent (B E P)
  (let ((x (make-extent (+ B tdb-offset) (+ E tdb-offset))))
    (set-extent-property x 'highlight t)
    (set-extent-property x 'tdb P)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; ancestral retry menu

; Initialize the extent for the ancestor entry

(defun tdb-ancestor (B E Invoc)
  (tdb-make-extent B E (cons 'retry Invoc)))

; retry what the user has indicated with the mouse

(defun tdb-retry (b Invoc)
  (goto-char (point-max))
  (insert "\n")
  (process-send-string (buffer-name b) (concat "r " Invoc "\n"))
  t)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; goto errors

(defun tdb-compilation-error (C File B E)
  (tdb-make-extent B E (list 'error C File)))

(defun tdb-error (l) (find-file (cadr l)) (goto-char (car l)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; respond to button clicks

(defun tdb-button (event)
  (interactive "e")
  (cond
   ((condition-case nil
	(let* ((b (window-buffer (event-window event)))
	       (x (extent-at (event-point event) b 'tdb)))
	  (cond
	   (x (let* ((l (extent-property x 'tdb))
		     (h (car l)) (Tail (cdr l)))
		(cond
		 ((eq h 'retry) (tdb-retry b Tail))
		 ((eq h 'error) (tdb-error Tail)))))))
      (error nil)))
   (t (mouse-yank event))))

(define-key inferior-prolog-mode-map 'button2 'tdb-button)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defconst tdb-was-loaded t)
